/*
 * Copyright (C) 2006 Michael Niedermayer <michaelni@gmx.at>
 *
 * This file is part of Libav.
 *
 * Libav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Libav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Libav; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

//#include "libavutil/cpu.h"
//#include "libavutil/x86_cpu.h"
//#include "libavcodec/x86/dsputil_mmx.h"
#include "yadif.h"

#ifdef __GNUC__
 #define DECLARE_ALIGNED(n,t,v)       t v __attribute__ ((aligned (n)))
 #define DECLARE_ASM_CONST(n,t,v)     static const t v __attribute__((used)) __attribute__ ((aligned (n)))
#else
 #define DECLARE_ALIGNED(n,t,v)      __declspec(align(n)) t v
 #define DECLARE_ASM_CONST(n,t,v)    __declspec(align(n)) static const t v
#endif

#if ARCH_X86_64 && defined(PIC)
#    define LOCAL_MANGLE(a) #a "(%%rip)"
#else
#    define LOCAL_MANGLE(a) #a
#endif
#define MANGLE(a) LOCAL_MANGLE(a)

#if ARCH_X86_64
#    define OPSIZE "q"
#    define REG_a "rax"
#    define REG_b "rbx"
#    define REG_c "rcx"
#    define REG_d "rdx"
#    define REG_D "rdi"
#    define REG_S "rsi"
#    define PTR_SIZE "8"
typedef int64_t x86_reg;

#    define REG_SP "rsp"
#    define REG_BP "rbp"
#    define REGBP   rbp
#    define REGa    rax
#    define REGb    rbx
#    define REGc    rcx
#    define REGd    rdx
#    define REGSP   rsp

#elif ARCH_X86_32

#    define OPSIZE "l"
#    define REG_a "eax"
#    define REG_b "ebx"
#    define REG_c "ecx"
#    define REG_d "edx"
#    define REG_D "edi"
#    define REG_S "esi"
#    define PTR_SIZE "4"
typedef int32_t x86_reg;

#    define REG_SP "esp"
#    define REG_BP "ebp"
#    define REGBP   ebp
#    define REGa    eax
#    define REGb    ebx
#    define REGc    ecx
#    define REGd    edx
#    define REGSP   esp
#else
typedef int x86_reg;
#endif

typedef struct { uint64_t a, b; } xmm_reg;

DECLARE_ASM_CONST(16, const xmm_reg, vdr_shd_pb_1) = {0x0101010101010101ULL, 0x0101010101010101ULL};
DECLARE_ASM_CONST(16, const xmm_reg, vdr_shd_pw_1) = {0x0001000100010001ULL, 0x0001000100010001ULL};


#if HAVE_SSSE3
#define COMPILE_TEMPLATE_SSE 1
#define COMPILE_TEMPLATE_SSSE3 1
#undef RENAME
#define RENAME(a) a ## _ssse3
#include "x86_yadif_template.c"
#undef COMPILE_TEMPLATE_SSSE3
#endif

#if HAVE_SSE
#undef RENAME
#define RENAME(a) a ## _sse2
#include "x86_yadif_template.c"
#undef COMPILE_TEMPLATE_SSE
#endif

#if HAVE_MMX
#undef RENAME
#define RENAME(a) a ## _mmx
#include "x86_yadif_template.c"
#endif
